@@ -100,12 +100,12 @@ st_buffer.sfc = function(x, dist, nQuadSegs = 30,
 			stop("package s2 required, please install it first")
 		if (inherits(dist, "units")) {
 			if (!inherits(try(units(dist) <- as_units("rad"), silent = TRUE), "try-error"))
-				return(st_as_sfc(s2::s2_buffer_cells(st_as_s2(x), dist, radius = 1, ...),
+				return(st_as_sfc(s2::s2_buffer_cells(x, dist, radius = 1, ...),
 					crs = st_crs(x)))
 			units(dist) = as_units("m") # make sure has dimension length, possibly convert
 			dist = drop_units(dist)
 		}
-		st_as_sfc(s2::s2_buffer_cells(st_as_s2(x), dist, ...), crs = st_crs(x))
+		st_as_sfc(s2::s2_buffer_cells(x, dist, ...), crs = st_crs(x))
 	} else {
 		if (longlat) {
 			warning("st_buffer does not correctly buffer longitude/latitude data")
@@ -175,8 +175,9 @@ st_boundary.sf = function(x) {
 #' @details \code{st_convex_hull} creates the convex hull of a set of points
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
-#' plot(st_convex_hull(nc))
-#' plot(nc, border = grey(.5))
+#' nc_g = st_geometry(nc)
+#' plot(st_convex_hull(nc_g))
+#' plot(nc_g, border = grey(.5), add = TRUE)
 st_convex_hull = function(x)
 	UseMethod("st_convex_hull")
 
@@ -214,7 +215,7 @@ st_simplify.sfc = function(x, preserveTopology = FALSE, dTolerance = 0.0) {
 			warning("argument preserveTopology is ignored")
 		if (! requireNamespace("s2", quietly = TRUE))
 			stop("package s2 required, please install it first")
-		st_as_sfc(s2::s2_simplify(st_as_s2(x), dTolerance), crs = st_crs(x))
+		st_as_sfc(s2::s2_simplify(x, dTolerance), crs = st_crs(x))
 	} else {
 		stopifnot(mode(preserveTopology) == 'logical')
 		if (ll)
@@ -257,10 +258,56 @@ st_triangulate.sfc = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
 
 #' @export
 st_triangulate.sf = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
-	st_geometry(x) <- st_triangulate(st_geometry(x), dTolerance, bOnlyEdges)
-	x
+	st_set_geometry(x, st_triangulate(st_geometry(x), dTolerance, bOnlyEdges))
 }
 
+#' @name geos_unary
+#' @export
+#' @details \code{st_inscribed_circle} returns the maximum inscribed circle for polygon geometries. 
+#' For \code{st_inscribed_circle}, if \code{nQuadSegs} is 0 a 2-point LINESTRING is returned with the
+#' center point and a boundary point of every circle, otherwise a circle (buffer) is returned where
+#' \code{nQuadSegs} controls the number of points per quadrant to approximate the circle.
+#' \code{st_inscribed_circle} requires GEOS version 3.9 or above
+#' @examples
+#' if (sf_extSoftVersion()["GEOS"] >= "3.9.0") {
+#'   nc_t = st_transform(nc, 'EPSG:2264')
+#'   x = st_inscribed_circle(st_geometry(nc_t))
+#'   plot(st_geometry(nc_t), asp = 1, col = grey(.9))
+#'   plot(x, add = TRUE, col = '#ff9999')
+#' }
+st_inscribed_circle = function(x, dTolerance, ...)
+	UseMethod("st_inscribed_circle")
+
+#' @export
+st_inscribed_circle.sfg = function(x, dTolerance, ...) {
+	get_first_sfg(st_inscribed_circle(st_sfc(x), dTolerance, ...))
+}
+
+#' @export
+st_inscribed_circle.sfc = function(x, dTolerance = sqrt(st_area(st_set_crs(x, NA_crs_)))/1000, ..., nQuadSegs = 30) {
+	if (CPL_geos_version() >= "3.9.0") {
+		if (isTRUE(st_is_longlat(x)))
+			warning("st_inscribed_circle does not work correctly for longitude/latitude data")
+		nQ = rep(nQuadSegs, length.out = length(x))
+		ret = st_sfc(CPL_geos_op("inscribed_circle", x, nQ, integer(0),
+			dTolerance = rep(as.double(dTolerance), length.out = length(x)), logical(0),
+			bOnlyEdges = as.integer(FALSE)))
+		if (any(nQuadSegs > 0)) {
+			pts = st_cast(ret, "POINT")
+			idx = seq(1, length(pts) * 2, by = 2)
+			ret = st_buffer(pts[idx], st_length(st_set_crs(ret, NA_crs_)), nQuadSegs = nQuadSegs)
+		} 
+		ret
+	} else
+		stop("for st_inscribed_circle, GEOS version 3.9.0 or higher is required")
+}
+
+#' @export
+st_inscribed_circle.sf = function(x, dTolerance, ...) {
+	st_set_geometry(x, st_inscribed_circle(st_geometry(x), dTolerance), ...)
+}
+
+
 #' @name geos_unary
 #' @export
 #' @param envelope object of class \code{sfc} or \code{sfg} containing a \code{POLYGON} with the envelope for a voronoi diagram; this only takes effect when it is larger than the default envelope, chosen when \code{envelope} is an empty polygon
@@ -288,6 +335,7 @@ st_triangulate.sf = function(x, dTolerance = 0.0, bOnlyEdges = FALSE) {
 #'  plot(pts["id"], pch = 16) # ID is color
 #'  plot(st_set_geometry(pts, "pols")["id"], xlim = c(0,1), ylim = c(0,1), reset = FALSE)
 #'  plot(st_geometry(pts), add = TRUE)
+#'  layout(matrix(1)) # reset plot layout
 #' }
 st_voronoi = function(x, envelope, dTolerance = 0.0, bOnlyEdges = FALSE)
 	UseMethod("st_voronoi")
@@ -368,8 +416,8 @@ st_line_merge.sf = function(x) {
 #' @export
 #' @details \code{st_centroid} gives the centroid of a geometry
 #' @examples
-#' plot(nc, axes = TRUE)
-#' plot(st_centroid(nc), add = TRUE, pch = 3)
+#' plot(nc_g, axes = TRUE)
+#' plot(st_centroid(nc_g), add = TRUE, pch = 3, col = 'red')
 #' mp = st_combine(st_buffer(st_sfc(lapply(1:3, function(x) st_point(c(x,x)))), 0.2 * 1:3))
 #' plot(mp)
 #' plot(st_centroid(mp), add = TRUE, col = 'red') # centroid of combined geometry
@@ -393,14 +441,21 @@ largest_ring = function(x) {
 
 #' @export
 st_centroid.sfc = function(x, ..., of_largest_polygon = FALSE) {
-	if (isTRUE(st_is_longlat(x)))
-		warning("st_centroid does not give correct centroids for longitude/latitude data")
 	if (of_largest_polygon) {
 		multi = which(sapply(x, inherits, what = "MULTIPOLYGON") & lengths(x) > 1)
 		if (length(multi))
 			x[multi] = largest_ring(x[multi])
 	}
-	st_sfc(CPL_geos_op("centroid", x, numeric(0), integer(0), numeric(0), logical(0)))
+	longlat = isTRUE(st_is_longlat(x))
+	if (longlat && sf_use_s2()) {
+		if (! requireNamespace("s2", quietly = TRUE))
+			stop("package s2 required, please install it first")
+		st_as_sfc(s2::s2_centroid(x), crs = st_crs(x))
+	} else { 
+		if (longlat)
+			warning("st_centroid does not give correct centroids for longitude/latitude data")
+		st_sfc(CPL_geos_op("centroid", x, numeric(0), integer(0), numeric(0), logical(0)))
+	}
 }
 
 #' @export
@@ -419,8 +474,8 @@ st_centroid.sf = function(x, ..., of_largest_polygon = FALSE) {
 #' @export
 #' @details \code{st_point_on_surface} returns a point guaranteed to be on the (multi)surface.
 #' @examples
-#' plot(nc, axes = TRUE)
-#' plot(st_point_on_surface(nc), add = TRUE, pch = 3)
+#' plot(nc_g, axes = TRUE)
+#' plot(st_point_on_surface(nc_g), add = TRUE, pch = 3, col = 'red')
 st_point_on_surface = function(x)
 	UseMethod("st_point_on_surface")
 
@@ -578,27 +633,26 @@ geos_op2_df = function(x, y, geoms) {
 
 # after checking identical crs,
 # call geos_op2 function op on x and y:
-geos_op2_geom = function(op, x, y, s2_model = "closed", ...) {
+# DE-9IM compliant should use model = "closed", more robust seems:
+geos_op2_geom = function(op, x, y, s2_model = "semi-open", ...) {
 	stopifnot(st_crs(x) == st_crs(y))
 	x = st_geometry(x)
 	y = st_geometry(y)
-	s2_model = switch(as.character(s2_model), OPEN = 0, SEMI_OPEN = 1, CLOSED = 2, -1)
 	longlat = isTRUE(st_is_longlat(x))
-	if (longlat && sf_use_s2() && s2_model >= 0) {
+	if (longlat && sf_use_s2()) {
 		if (! requireNamespace("s2", quietly = TRUE))
 			stop("package s2 required, please install it first")
 		fn = switch(op, intersection = s2::s2_intersection,
 				difference = s2::s2_difference,
 				sym_difference = s2::s2_sym_difference,
 				union = s2::s2_union, stop("invalid operator"))
-		s2x = st_as_s2(x)
-		s2y = st_as_s2(y)
 		# to be optimized -- this doesn't index on y:
-		lst = structure(unlist(lapply(s2x, fn, s2y, s2::s2_options(model = s2_model, ...)),
-			recursive = FALSE), class = class(s2x))
+		lst = structure(unlist(lapply(x, fn, y, s2::s2_options(model = s2_model, ...)),
+			recursive = FALSE), class = "s2_geography")
 		e = s2::s2_is_empty(lst)
 		idx = cbind(rep(seq_along(x), length(y)), rep(seq_along(y), each = length(x)))
-		structure(st_as_sfc(lst[!e], crs = st_crs(x)), idx = idx[!e,,drop = FALSE])
+		lst = st_as_sfc(lst, crs = st_crs(x))
+		structure(lst[!e], idx = idx[!e,,drop = FALSE])
 	} else {
 		if (longlat)
 			message_longlat(paste0("st_", op))
@@ -814,16 +868,23 @@ st_union.sfg = function(x, y, ..., by_feature = FALSE, is_coverage = FALSE) {
 
 #' @export
 st_union.sfc = function(x, y, ..., by_feature = FALSE, is_coverage = FALSE) {
-	if (missing(y)) # unary union, possibly by_feature:
-		st_sfc(CPL_geos_union(st_geometry(x), by_feature, is_coverage))
-	else
+	if (missing(y)) { # unary union, possibly by_feature:
+		ll = isTRUE(st_is_longlat(x))
+		if (ll && sf_use_s2() && !by_feature)
+			st_as_sfc(s2::s2_union_agg(x, ...), crs = st_crs(x))
+		else {
+			if (ll)
+				message_longlat("st_union")
+			st_sfc(CPL_geos_union(st_geometry(x), by_feature, is_coverage))
+		}
+	} else
 		geos_op2_geom("union", x, y)
 }
 
 #' @export
 st_union.sf = function(x, y, ..., by_feature = FALSE, is_coverage = FALSE) {
 	if (missing(y)) { # unary union, possibly by_feature:
-		geom = st_sfc(CPL_geos_union(st_geometry(x), by_feature, is_coverage))
+		geom = st_union(st_geometry(x), ..., by_feature = by_feature, is_coverage = is_coverage)
 		if (by_feature)
 			st_set_geometry(x, geom)
 		else
