@@ -1,4 +1,4 @@
-#' @importFrom utils head object.size str tail
+#' @importFrom utils head object.size str tail packageVersion
 #' @importFrom stats aggregate na.omit rbinom runif setNames
 #' @importFrom tools file_ext file_path_sans_ext
 #' @importFrom methods as new slot slotNames
@@ -35,8 +35,8 @@ pathGrob <- NULL
 		pathGrob <<- function(..., pathId.lengths) {
 			grid::pathGrob(...)
 		}
-	}
-	load_gdal() # nocov end
+	} # nocov end
+	load_gdal() 
 	assign(".sf.use_s2", Sys.getenv("_SF_USE_S2") == "true", envir=.sf_cache)
 }
 
