@@ -3,10 +3,12 @@
 # https://github.com/r-spatial/s2
 
 #' @export
+#' @param crs coordinate reference system; object of class \code{crs}
+#' @param ... passed on
 #' @param use_s2 logical; if \code{TRUE}, use the s2 spherical geometry package
 #' for geographical coordinate operations
 #' @name s2
-#' @return \code{sf_use_s2} returns the value of this variable before (re)setting it, 
+#' @return \code{sf_use_s2} returns the value of this variable before (re)setting it,
 #' invisibly if \code{use_s2} is not missing.
 sf_use_s2 = function(use_s2) {
 	ret_val = get(".sf.use_s2", envir = .sf_cache)
@@ -24,16 +26,6 @@ sf_use_s2 = function(use_s2) {
 		ret_val
 }
 
-# from s2 to sf:
-#' @export
-#' @param crs coordinate reference system; object of class \code{crs}
-#' @param ... passed on
-#' @name s2
-st_as_sfc.wk_wkb = function(x, ..., crs = st_crs(4326)) {
-	class(x) = "WKB"
-	st_set_crs(st_as_sfc(x, ...), crs)
-}
-
 
 #' @name s2
 #' @export
@@ -51,11 +43,20 @@ st_as_sf.s2_geography = function(x, ..., crs = st_crs(4326)) {
 	st_sf(geometry = st_as_sfc(x, ..., crs = crs))
 }
 
+# dynamically exported in tidyverse.R
+as_s2_geography.sfc <- function(x, ..., oriented = FALSE) {
+	st_as_s2.sfc(x, ..., oriented = oriented)
+}
+
+# dynamically exported in tidyverse.R
+as_s2_geography.sf <- function(x, ..., oriented = FALSE) {
+	st_as_s2.sf(x, ..., oriented = oriented)
+}
 
 #' functions for spherical geometry, using s2 package
-#' 
+#'
 #' functions for spherical geometry, using the s2 package based on the google s2geometry.io library
-#' @name s2 
+#' @name s2
 #' @param x object of class \code{sf}, \code{sfc} or \code{sfg}
 #' @export
 #' @details \code{st_as_s2} converts an \code{sf} POLYGON object into a form readable by \code{s2}.
@@ -65,8 +66,8 @@ st_as_sf.s2_geography = function(x, ..., crs = st_crs(4326)) {
 #' m0 = m[5:1,]
 #' mp = st_multipolygon(list(
 #'	list(m, 0.8 * m0, 0.01 * m1 + 0.9),
-#'	list(0.7* m, 0.6*m0), 
-#'	list(0.5 * m0), 
+#'	list(0.7* m, 0.6*m0),
+#'	list(0.5 * m0),
 #'	list(m+2),
 #'	list(m+4,(.9*m0)+4)
 #'	))
@@ -87,6 +88,8 @@ st_as_s2.sf = function(x, ...) st_as_s2(st_geometry(x), ...)
 st_as_s2.sfc = function(x, ..., oriented = FALSE) {
 	if (! requireNamespace("s2", quietly = TRUE))
 		stop('package s2 required, please install it first')
+	if (!is.na(st_crs(x)) && !st_is_longlat(x))
+		x = st_transform(x, ifelse(st_axis_order(), "OGC:CRS84", "EPSG:4326"))
 	if (length(x) && nchar(class(x[[1]])[1]) > 2) { # Z, M, ZM:
 		message("st_as_s2(): dropping Z and/or M coordinate")
 		x = st_zm(x)
