@@ -77,6 +77,13 @@ ungroup.sf <- function(x, ...) {
 	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
 }
 
+#' @name tidyverse
+rowwise.sf <- function(x, ...) {
+	sf_column_name = attr(x, "sf_column")
+	class(x) <- setdiff(class(x), "sf")
+	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
+}
+
 .re_sf = function(x, sf_column_name, agr, geom = NULL) {
 	stopifnot(!inherits(x, "sf"), !missing(sf_column_name), !missing(agr))
 	# non-geom attribute names
@@ -137,16 +144,17 @@ select.sf <- function(.data, ...) {
 
 	agr = st_agr(.data)
 	vars = names(.data)[setdiff(loc, sf_column_loc)]
-	new_agr = agr[vars]
 
 	sf_column_loc_loc = match(sf_column_loc, loc)
 	if (is.na(sf_column_loc_loc)) {
 		# The sf column was subsetted out, select it back in
+		new_agr = setNames(agr[vars], names(loc))
 		loc = c(loc, sf_column_loc)
 		names(loc)[[length(loc)]] = sf_column
 	} else {
 		# The sf column was not subsetted out but it might have been renamed
 		sf_column = names(loc[sf_column_loc_loc])
+		new_agr = setNames(agr[vars], setdiff(names(loc), sf_column))
 	}
 
 	ret = .data
@@ -205,6 +213,7 @@ slice.sf <- function(.data, ..., .dots) {
 #' @name tidyverse
 #' @aliases summarise
 #' @param do_union logical; in case \code{summary} does not create a geometry column, should geometries be created by unioning using \link{st_union}, or simply by combining using \link{st_combine}? Using \link{st_union} resolves internal boundaries, but in case of unioning points, this will likely change the order of the points; see Details.
+#' @param is_coverage logical; if \code{do_union} is \code{TRUE}, use an optimized algorithm for features that form a polygonal coverage (have no overlaps)
 #' @return an object of class \link{sf}
 #' @details
 #' In case one or more of the arguments (expressions) in the \code{summarise} call creates a geometry list-column, the first of these will be the (active) geometry of the returned object. If this is not the case, a geometry column is created, depending on the value of \code{do_union}.
@@ -216,7 +225,7 @@ slice.sf <- function(.data, ..., .dots) {
 #' nc.g %>% summarise(mean(AREA))
 #' nc.g %>% summarise(mean(AREA)) %>% plot(col = grey(3:6 / 7))
 #' nc %>% as.data.frame %>% summarise(mean(AREA))
-summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
+summarise.sf <- function(.data, ..., .dots, do_union = TRUE, is_coverage = FALSE) {
 	sf_column = attr(.data, "sf_column")
 	precision = st_precision(.data)
 	crs = st_crs(.data)
@@ -225,6 +234,8 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
 	ret = NextMethod()
 	if (!missing(do_union))
 		ret$do_union = NULL
+	if (!missing(is_coverage))
+		ret$is_coverage = NULL
 
 	if (! any(sapply(ret, inherits, what = "sfc"))) {
 		geom = if (inherits(.data, "grouped_df") || inherits(.data, "grouped_dt")) {
@@ -233,7 +244,7 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
 				i = dplyr::group_indices(.data)
 				# geom = st_geometry(.data)
 				geom = if (do_union)
-						lapply(sort(unique(i)), function(x) st_union(geom[i == x]))
+						lapply(sort(unique(i)), function(x) st_union(geom[i == x], is_coverage = is_coverage))
 					else
 						lapply(sort(unique(i)), function(x) st_combine(geom[i == x]))
 				geom = unlist(geom, recursive = FALSE)
@@ -242,7 +253,7 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE) {
 				do.call(st_sfc, c(geom, crs = list(crs), precision = precision))
 			} else { # single group:
 				if (do_union)
-					st_union(geom)
+					st_union(geom, is_coverage = is_coverage)
 				else
 					st_combine(geom)
 			}
@@ -272,9 +283,9 @@ distinct.sf <- function(.data, ..., .keep_all = FALSE) {
 	if (!requireNamespace("rlang", quietly = TRUE))
 		stop("rlang required: install first?")
 
-	.data = dplyr::distinct(.data, ..., !! rlang::sym(sf_column), .keep_all = .keep_all)
+	.data = dplyr::distinct(.data, ..., .keep_all = .keep_all)
 	.data[[ sf_column ]] = geom[ .data[[ sf_column ]] ]
-	st_as_sf(.data)
+	st_as_sf(.data, sf_column_name = sf_column)
 }
 
 ## tidyr methods:
@@ -480,6 +491,7 @@ register_all_s3_methods = function() {
 	register_s3_method("dplyr", "mutate", "sf")
 	register_s3_method("dplyr", "rename", "sf")
 	register_s3_method("dplyr", "right_join", "sf")
+	register_s3_method("dplyr", "rowwise", "sf")
 	register_s3_method("dplyr", "sample_frac", "sf")
 	register_s3_method("dplyr", "sample_n", "sf")
 	register_s3_method("dplyr", "select", "sf")
@@ -498,19 +510,21 @@ register_all_s3_methods = function() {
 	register_s3_method("pillar", "obj_sum", "sfc")
 	register_s3_method("pillar", "type_sum", "sfc")
 	register_s3_method("pillar", "pillar_shaft", "sfc")
-	register_s3_method("spatstat", "as.ppp", "sfc")
-	register_s3_method("spatstat", "as.ppp", "sf")
-	register_s3_method("spatstat", "as.owin", "POLYGON")
-	register_s3_method("spatstat", "as.owin", "MULTIPOLYGON")
-	register_s3_method("spatstat", "as.owin", "sfc_POLYGON")
-	register_s3_method("spatstat", "as.owin", "sfc_MULTIPOLYGON")
-	register_s3_method("spatstat", "as.owin", "sfc")
-	register_s3_method("spatstat", "as.owin", "sf")
-	register_s3_method("spatstat", "as.psp", "LINESTRING")
-	register_s3_method("spatstat", "as.psp", "MULTILINESTRING")
-	register_s3_method("spatstat", "as.psp", "sfc_MULTILINESTRING")
-	register_s3_method("spatstat", "as.psp", "sfc")
-	register_s3_method("spatstat", "as.psp", "sf")
+	register_s3_method("spatstat.geom", "as.ppp", "sfc")
+	register_s3_method("spatstat.geom", "as.ppp", "sf")
+	register_s3_method("spatstat.geom", "as.owin", "POLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "MULTIPOLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "sfc_POLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "sfc_MULTIPOLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "sfc")
+	register_s3_method("spatstat.geom", "as.owin", "sf")
+	register_s3_method("spatstat.geom", "as.psp", "LINESTRING")
+	register_s3_method("spatstat.geom", "as.psp", "MULTILINESTRING")
+	register_s3_method("spatstat.geom", "as.psp", "sfc_MULTILINESTRING")
+	register_s3_method("spatstat.geom", "as.psp", "sfc")
+	register_s3_method("spatstat.geom", "as.psp", "sf")
+	register_s3_method("s2", "as_s2_geography", "sfc")
+	register_s3_method("s2", "as_s2_geography", "sf")
 	register_vctrs_methods()
 }
 
