@@ -75,14 +75,14 @@ List CPL_get_crs(CharacterVector obj, CharacterVector options) {
 	NumericVector gt_r(6);
 	for (int i = 0; i < 6; i++)
 		gt_r(i) = gt[i];
-	ret(2) =  gt_r;
+	ret(2) = gt_r;
 
 	double gt_inv[6];
 	int retval = GDALInvGeoTransform(gt, gt_inv);
 	NumericVector gt_r_inv(6);
 	for (int i = 0; i < 6; i++)
 		gt_r_inv(i) = retval ? gt_inv[i] : NA_REAL;
-	ret(3) =  gt_r_inv;
+	ret(3) = gt_r_inv;
 
 	ret.attr("names") = CharacterVector::create("nbands", "crs", "gt", "gt_inv");
 
@@ -165,19 +165,17 @@ NumericVector read_gdal_data(GDALDataset *poDataset,
 		if (has_offset)
 			offset = poBand->GetOffset(NULL);
 		units[i] = poBand->GetUnitType();
-		if (! NumericVector::is_na(nodatavalue[0]) || has_offset || has_scale) {
-
-
-			// for (R_xlen_t j = 0; j < Rf_xlength(vec); j++)
-			for (R_xlen_t j = i * (((R_xlen_t) nBufXSize) * nBufYSize); // start of band i
-					j < (i + 1) * (((R_xlen_t) nBufXSize) * nBufYSize); // end of band i
-					j++) {
-				if (equals_na(vec[j], nodatavalue[0], poBand->GetRasterDataType()))
-					vec[j] = NA_REAL; // #nocov
-				else
-					vec[j] = (vec[j] * scale) + offset;
-			}
+		// if (! NumericVector::is_na(nodatavalue[0]) || has_offset || has_scale) { 
+		// outcommented because of NaN handling, https://github.com/r-spatial/stars/issues/333
+		for (R_xlen_t j = i * (((R_xlen_t) nBufXSize) * nBufYSize); // start of band i
+				j < (i + 1) * (((R_xlen_t) nBufXSize) * nBufYSize); // end of band i
+				j++) {
+			if (equals_na(vec[j], nodatavalue[0], poBand->GetRasterDataType()))
+				vec[j] = NA_REAL; // #nocov
+			else
+				vec[j] = (vec[j] * scale) + offset;
 		}
+		// }
 		checkUserInterrupt();
 	}
 	vec.attr("units") = units;
@@ -196,7 +194,7 @@ NumericVector read_gdal_data(GDALDataset *poDataset,
 }
 
 int get_from_list(List lst, const char *name, int otherwise) {
-	if (lst.containsElementNamed(name)) {
+	if (lst.containsElementNamed(name) && lst[name] != R_NilValue) {
 		IntegerVector ret = lst[name]; // #nocov
 		return(ret[0]);                // #nocov
 	} else
@@ -288,18 +286,18 @@ List get_rat(GDALRasterAttributeTable *tbl) {
 List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVector driver,
 		bool read_data, NumericVector NA_value, List RasterIO_parameters) {
 // reads and returns data set metadata, and if read_data is true, adds data array
-    GDALDataset *poDataset = (GDALDataset *) GDALOpenEx(fname[0], GA_ReadOnly,
+	GDALDataset *poDataset = (GDALDataset *) GDALOpenEx(fname[0], GA_ReadOnly,
 		driver.size() ? create_options(driver).data() : NULL,
 		options.size() ? create_options(options).data() : NULL,
 		NULL);
-    if (poDataset == NULL) {
+	if (poDataset == NULL) {
 		Rcout << "trying to read file: " << fname[0] << std::endl; // #nocov
-        stop("file not found"); // #nocov
+		stop("file not found"); // #nocov
 	}
 
 	CharacterVector Driver = CharacterVector::create(
-        poDataset->GetDriver()->GetDescription(),
-        poDataset->GetDriver()->GetMetadataItem( GDAL_DMD_LONGNAME ));
+		poDataset->GetDriver()->GetDescription(),
+		poDataset->GetDriver()->GetMetadataItem( GDAL_DMD_LONGNAME ));
 
 	/*
 	if (poDataset->GetRasterCount() == 0)
@@ -351,18 +349,29 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 			nodatavalue[0] = poBand->GetNoDataValue(NULL); // #nocov
 	}
 
+	// bands:
+	IntegerVector bands;
+	if (RasterIO_parameters.containsElementNamed("bands"))
+		bands = RasterIO_parameters["bands"]; // #nocov
+	else {
+		bands = IntegerVector(poDataset->GetRasterCount());
+		for (int j = 0; j < bands.size(); j++)
+			bands(j) = j + 1; // bands is 1-based
+	}
+
 	// get color table, attribute table, and min/max values:
-	List colorTables(poDataset->GetRasterCount());
-	List attributeTables(poDataset->GetRasterCount());
-	CharacterVector descriptions(poDataset->GetRasterCount());
-	NumericMatrix ranges(poDataset->GetRasterCount(), 4);
-	for (int i = 0; i < poDataset->GetRasterCount(); i++) {
-		poBand = poDataset->GetRasterBand(i + 1);
+	List colorTables(bands.size());
+	List attributeTables(bands.size());
+	CharacterVector descriptions(bands.size());
+	NumericMatrix ranges(bands.size(), 4);
+	for (int i = 0; i < bands.size(); i++) {
+		poBand = poDataset->GetRasterBand(bands(i));
 		const char *md = poBand->GetMetadataItem("BANDNAME", NULL);
 		if (md == NULL)
 			descriptions(i) = poBand->GetDescription();
 		else
 			descriptions(i) = md;
+
 		if (poBand->GetColorTable() != NULL)
 			colorTables(i) = get_color_table(poBand->GetColorTable());
 		if (poBand->GetCategoryNames() != NULL)
@@ -393,16 +402,6 @@ List CPL_read_gdal(CharacterVector fname, CharacterVector options, CharacterVect
 	int nBufXSize = get_from_list(RasterIO_parameters, "nBufXSize", nXSize);
 	int nBufYSize = get_from_list(RasterIO_parameters, "nBufYSize", nYSize);
 
-	// bands:
-	IntegerVector bands;
-	if (RasterIO_parameters.containsElementNamed("bands"))
-		bands = RasterIO_parameters["bands"]; // #nocov
-	else {
-		bands = IntegerVector(poDataset->GetRasterCount());
-		for (int j = 0; j < bands.size(); j++)
-			bands(j) = j + 1; // bands is 1-based
-	}
-
 	// resampling method:
 	GDALRasterIOExtraArg resample;
 	INIT_RASTERIO_EXTRA_ARG(resample);
@@ -570,6 +569,46 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 				}
 			}
 		}
+		// set descriptions:
+		if (x.attr("descriptions") != R_NilValue) {
+			Rcpp::CharacterVector descriptions = x.attr("descriptions");
+			for (int band = 1; band <= dims(2); band++) {
+				GDALRasterBand *poBand = poDstDS->GetRasterBand( band );
+				poBand->SetDescription(descriptions(band-1));
+			}
+		}
+
+		// write factor levels to CategoryNames:
+		if (x.attr("levels") != R_NilValue) {
+			Rcpp::CharacterVector levels = x.attr("levels");
+			Rcpp::CharacterVector l(levels.size() + 1);
+			l[0] = ""; // levels start at 1, CategoryNames at 0.
+			for (int i = 0; i < levels.size(); i++)
+				l[i+1] = levels[i];
+			for (int band = 1; band <= dims(2); band++) {
+				GDALRasterBand *poBand = poDstDS->GetRasterBand( band );
+				if (poBand->SetCategoryNames(create_options(l).data()) != CE_None)
+					warning("error writing factor levels to raster band");
+			}
+		}
+		// write color table:
+		if (x.attr("rgba") != R_NilValue) {
+			Rcpp::NumericMatrix co = x.attr("rgba"); // r g b alpha in columns; levels in rows
+			GDALColorTable ct = GDALColorTable(GPI_RGB);
+			GDALColorEntry ce;
+			ce.c1 = 0; ce.c2 = 0; ce.c3 = 0; ce.c4 = 0;
+			ct.SetColorEntry(0, &ce);
+			for (int i = 0; i < co.nrow(); i++) {
+				ce.c1 = co(i, 0);
+				ce.c2 = co(i, 1);
+				ce.c3 = co(i, 2);
+				ce.c4 = co(i, 3);
+				ct.SetColorEntry(i + 1, &ce);
+			}
+			GDALRasterBand *poBand = poDstDS->GetRasterBand( 1 ); // can only set CT for band 1
+			if (poBand->SetColorTable(&ct) != CE_None)
+				warning("error writing color table to raster band");
+		}
 
 	} else { // no create, update:
 		if ((poDstDS = (GDALDataset *) GDALOpen(fname[0], GA_Update)) == NULL) // #nocov
@@ -600,3 +639,113 @@ void CPL_write_gdal(NumericMatrix x, CharacterVector fname, CharacterVector driv
 	GDALClose( (GDALDatasetH) poDstDS );
 	return;
 }
+
+double get_bilinear(GDALRasterBand *poBand, double Pixel, double Line, 
+						int iPixel, int iLine, double RasterXSize, double RasterYSize,
+						int na_set, double na_value) {
+
+	double pixels[4];
+	double dY  = Line - iLine; // [0, 1) over a raster cell
+	double dX  = Pixel - iPixel; // [0, 1) over a raster cell
+	if ((dY < 0.5 && iLine > 0) || (iLine == RasterYSize - 1)) // where to start reading
+		iLine -= 1;
+	if ((dX < 0.5 && iPixel > 0) || (iPixel == RasterXSize - 1))
+		iPixel -= 1;
+
+	// x:
+	if (Pixel < 0.5) // border:
+		dX = 0.0;
+	else if (Pixel > RasterXSize - 0.5)
+		dX = 1.0;
+	else if (dX < 0.5) // shift to pixel center:
+		dX += 0.5;
+	else
+		dX -= 0.5;
+
+	// y:
+	if (Line < 0.5)
+		dY = 0.0;
+	else if (Line > RasterYSize - 0.5)
+		dY = 1.0;
+	else if (dY < 0.5)
+		dY += 0.5;
+	else
+		dY -= 0.5;
+
+	// read:
+	if (GDALRasterIO(poBand, GF_Read, iPixel, iLine, 2, 2,
+			(void *) pixels, 2, 2, GDT_CFloat64, sizeof(double), 0) != CE_None)
+		stop("Error reading!");
+	// f(0,0): pixels[0], f(1,0): pixels[1], f(0,1): pixels[2], f(1,1): pixels[3]
+	if (na_set && (pixels[0] == na_value || pixels[1] == na_value ||
+			pixels[2] == na_value || pixels[3] == na_value))
+		return na_value;
+	else // https://en.wikipedia.org/wiki/Bilinear_interpolation#Unit_square
+		return	pixels[0] * (1-dX) * (1-dY) +
+				pixels[1] * dX     * (1-dY) +
+				pixels[2] * (1-dX) * dY +
+				pixels[3] * dX     * dY;
+}
+
+// [[Rcpp::export]]
+NumericMatrix CPL_extract(CharacterVector input, NumericMatrix xy, bool interpolate = false) {
+	// mostly taken from gdal/apps/gdallocationinfo.cpp
+
+	GDALDataset *poDataset = (GDALDataset *) GDALOpenEx(input[0], GA_ReadOnly,
+		NULL, NULL, NULL);
+	if (poDataset == NULL) {
+		Rcout << "trying to read file: " << input[0] << std::endl; // #nocov
+		stop("file not found"); // #nocov
+	}
+
+	NumericMatrix ret(xy.nrow(), poDataset->GetRasterCount());
+
+	double gt[6];
+	poDataset->GetGeoTransform(gt);
+	double gt_inv[6];
+	// int retval = GDALInvGeoTransform(gt, gt_inv);
+	if (! GDALInvGeoTransform(gt, gt_inv))
+		stop("geotransform not invertible");
+
+	for (int j = 0; j < poDataset->GetRasterCount(); j++) {
+		GDALRasterBand *poBand = poDataset->GetRasterBand(j+1);
+		int bSuccess;
+		double dfOffset = poBand->GetOffset(&bSuccess);
+		double dfScale  = poBand->GetScale(&bSuccess);
+		double nodata = NA_REAL;
+		int nodata_set = 0;
+		poBand->GetNoDataValue(&nodata_set);
+		if (nodata_set)
+			nodata = poBand->GetNoDataValue(NULL);
+		for (int i = 0; i < xy.nrow(); i++) {
+			double dfGeoX = xy(i, 0);
+			double dfGeoY = xy(i, 1);
+			double Pixel = gt_inv[0] + gt_inv[1] * dfGeoX + gt_inv[2] * dfGeoY;
+			double Line  = gt_inv[3] + gt_inv[4] * dfGeoX + gt_inv[5] * dfGeoY;
+			int iPixel = static_cast<int>(floor( Pixel ));
+			int iLine  = static_cast<int>(floor( Line ));
+			double pixel;
+			if (iPixel < 0 || iLine < 0 ||
+					iPixel >= poDataset->GetRasterXSize() ||
+					iLine  >= poDataset->GetRasterYSize()) // outside bbox:
+				pixel = NA_REAL;
+			else { // read pixel:
+				if (interpolate)
+					// stop("interpolate not implemented");
+					pixel = get_bilinear(poBand, Pixel, Line, iPixel, iLine,
+						poDataset->GetRasterXSize(), poDataset->GetRasterYSize(),
+						nodata_set, nodata);
+				else if (GDALRasterIO(poBand, GF_Read, iPixel, iLine, 1, 1,
+						&pixel, 1, 1, GDT_CFloat64, 0, 0) != CE_None)
+					stop("Error reading!");
+				if (nodata_set && pixel == nodata)
+					pixel = NA_REAL;
+				else if (dfOffset != 0.0 || dfScale != 1.0)
+					pixel = pixel * dfScale + dfOffset;
+			}
+			ret(i, j) = pixel;
+		}
+	}
+	GDALClose(poDataset);
+	return ret;
+}
